from __future__ import absolute_import, division, print_function
from __future__ import unicode_literals

import argparse
import os
import sys

from .commands import merge_pr
from .commands import deploy
from .commands import create_branch
from .commands import open_pr
from .commands import lock_project
from .commands import ask_review
from .commands import create_qa
from .commands import delete_qa
from .lib.handler import prompt_utils
from .lib.config import Config

config = Config().load()

parser = argparse.ArgumentParser()
parser.add_argument(
    "--project", help="Nome del progetto su cui eseguire il comando (default directory corrente)", default=os.path.basename(os.getcwd()))

subparsers = parser.add_subparsers(dest="command", help="sub-command help")

parser_create_branch = subparsers.add_parser(
    "create-branch", help="Crea branch locale e imposta la card di YouTrack in progress")
parser_create_branch.add_argument(
    "--card", help="Numero card youtrack (ex. PRIMA-123)")

parser_lock_project = subparsers.add_parser(
    "lock-project", help="Lock/unlock dei merge sul branch master di un progetto")
parser_lock_project.add_argument(
    "action", choices=["lock", "l", "unlock", "u"])

parser_open_pr = subparsers.add_parser(
    "open-pr", help="Apre una PR su GitHub")

parser_ask_review = subparsers.add_parser(
    "ask-review", help="Chiede la review di una PR")

parser_create_qa = subparsers.add_parser(
    "create-qa", help="Crea un QA (integrazione con qainit)")

parser_delete_qa = subparsers.add_parser(
    "delete-qa", help="Cancella un QA (integrazione con qainit)")

parser_merge_pr = subparsers.add_parser(
    "merge-pr", help="Merge del branch selezionato con master se tutti i check sono ok")

parser_deploy = subparsers.add_parser(
    "deploy", help="Deploy in produzione del branch master")


def dispatch(cmd_args):
    args = parser.parse_args(args=cmd_args)

    if not prompt_utils.ask_confirm("Vuoi continuare sul progetto {}?".format(args.project)):
        sys.exit(-1)

    os.chdir(os.path.join(config['user']['projects_home'], args.project))

    if(args.command == "create-branch"):
        return create_branch.entrypoint(args)
    elif(args.command == "deploy"):
        return deploy.entrypoint(args)
    elif(args.command == "merge-pr"):
        return merge_pr.entrypoint(args)
    elif(args.command == "open-pr"):
        return open_pr.entrypoint(args)
    elif(args.command == "lock-project"):
        return lock_project.entrypoint(args)
    elif(args.command == "ask-review"):
        return ask_review.entrypoint(args)
    elif(args.command == "create-qa"):
        return create_qa.entrypoint(args)
    elif(args.command == "delete-qa"):
        return delete_qa.entrypoint(args)



from __future__ import print_function, unicode_literals
from ..lib.handler.youtrack_handler import YoutrackHandler
from ..lib.handler.github_handler import GithubHandler, GithubException
from ..lib.logger import Logger
from ..lib.config import Config
from ..lib.handler import git_handler as git
from ..lib.handler import prompt_utils
import sys

youtrack = YoutrackHandler()
github = GithubHandler()
config = Config().load()
logger = Logger()


def entrypoint(args):
    branch_name = git.current_branch_name()
    git.is_dirty(args.project)

    if not git.remote_branch_exists(args.project, branch_name):
        logger.error("Non ho trovato il branch {} su GitHub, per favore esegui un 'git push' manualmente".format(branch_name))
        sys.exit(-1)

    youtrack_id = youtrack.get_card_from_name(branch_name)
    if youtrack_id:
        pulls = github.get_pr_from_branch(args.project, branch_name)
        if pulls.totalCount:
            pr = pulls[0]
            logger.info("Esiste una pull request su GitHub per il branch {}".format(branch_name))

            if prompt_utils.ask_confirm("Vuoi modificare la description della pull request?"):
                edit_pr(pr)
                sys.exit(0)

        else:
            create_pr(args.project, branch_name, youtrack_id)
    else:
        logger.warning("Non sono riuscito a trovare una issue YouTrack nel nome del branch o la issue indicata non esiste")
        if not prompt_utils.ask_confirm("Vuoi collegare la pull request con una issue?"):
            create_pr(args.project, branch_name)
        else:
            create_pr(args.project, branch_name, ask_for_id_card())


def edit_pr(pr):
    pr_body = ask_for_description(pr.body)
    pr.edit(body=pr_body)
    logger.info("Pull request modificata")


def create_pr(repo, branch_name, id_card=None):
    if id_card:
        logger.info("Creo una pull request sul progetto {} per il branch {} collegato con la card {}".format(repo, branch_name, id_card))
        link = youtrack.get_link(id_card)
        title = "[{}]: {}".format(
            id_card, youtrack.get_issue(id_card)["summary"])
    else:
        logger.warning("Creo una pull request sul progetto {} per il branch {} SENZA collegamenti a YouTrack".format(
            repo, branch_name))
        link = ""
        title = ask_for_title()

    pr_body = ask_for_description()

    body = "{} \n\n {}".format(link, pr_body)

    try:
        pr = github.create_pr(repo, branch_name, title, body)
        logger.info("Pull request numero {} creata!".format(pr.number))
    except GithubException as e:
        logger.error('Errore durante la richiesta a GitHub: ')
        logger.error(e.data["errors"][0])
        sys.exit(-1)

    if id_card:
        update_card(id_card, repo, pr.html_url)
        logger.info("Inserito link della pull request nella card {}".format(id_card))


def ask_for_description(pr_body=""):
    input("Inserisci la description della pull request. Premendo invio si aprira l'editor di default")
    description = prompt_utils.ask_questions_editor(
        'Inserisci la description della PR: ', pr_body)

    if description == "":
        logger.warning("La descrizione della pull request non può essere vuota")
        ask_for_description(pr_body)
    else:
        return description


def ask_for_id_card():
    id_card = prompt_utils.ask_questions_input('Inserisci ID della card (ex: PRIMA-1234): ')
    if youtrack.validate_issue(id_card):
        return id_card
    else:
        logger.error("ID non esistente su YouTrack")
        ask_for_id_card()


def update_card(id_card, repo, link):
    youtrack.comment(id_card, "PR {} -> {}".format(repo, link))


def ask_for_title():
    return prompt_utils.ask_questions_input('Inserisci il titolo della pull request: ')



from __future__ import print_function, unicode_literals
from .logger import Logger
from .config import Config
from .handler import git_handler as git
from .handler.github_handler import GithubHandler

import subprocess
import json

config = Config().load()
logger = Logger()
github = GithubHandler()

# development only
#twig_command = '{}/twig-binaries/bin/twig-feature'.format(config['user']['projects_home'])

qainit_dir = '{}/qainit'.format(config['user']['projects_home'])


def get_qa_projects():
    git.check_repo_cloned('qainit')
    git.sync('qainit')
    with open('{}/branch_names'.format(qainit_dir), 'r') as file:
        branches_obj = json.loads(file.read())

    return list(branches_obj.keys())


def qainit_deploy(args):
    return subprocess.run(
        # development only
        #[twig_command, 'suite', 'deploy', args], cwd=qainit_dir, check=True)
        ['twig', 'feature', 'suite', 'deploy', args], cwd=qainit_dir, check=True)


def qainit_shutdown(youtrack_id):
    git.check_repo_cloned('qainit')
    git.sync('qainit')
    branch = git.search_remote_branch('qainit', '*{}*'.format(youtrack_id))
    if branch:
        try:
            git.checkout('qainit', branch)
            git.commit('qainit', 'shutdown', dummy=True)
            git.push('qainit', branch)
        except:
            logger.error(
                "Non sono riuscito a spegnere il qa, per favore spegnilo manualmente!\nDevops are watching you! ( •͡˘ _•͡˘)")

    else:
        logger.warning(
            "Non sono riuscito a trovare un qa per questa issue, se il qa esiste, per favore spegnilo manualmente\nDevops are watching you! ( •͡˘ _•͡˘)")



from PyInquirer import prompt
import sys


def ask_confirm(text, default=True):
    questions = [
        {
            'type': 'confirm',
            'message': text,
            'name': 'continue',
            'default': default,
        }
    ]
    answer = prompt(questions)
    if answer["continue"]:
        return True
    else:
        return False


def ask_questions_input(question_text, default_text=""):
    questions = [
        {
            'type': 'input',
            'name': 'value',
            'message': question_text,
            'default': default_text,
        }
    ]

    answer = prompt(questions)
    return answer['value']


def ask_questions_editor(question_text, default_text=""):
    questions = [
        {
            'type': 'editor',
            'name': 'value',
            'message': question_text,
            'default': default_text,
            'eargs': {
                'editor': 'default'
            }
        }
    ]

    answer = prompt(questions)
    return answer['value']


def ask_choices(question_text, choices, default_text=""):
    questions = [
        {
            'type': 'list',
            'name': 'value',
            'message': question_text,
            'choices': choices,
            'default': default_text,
        }
    ]

    answer = prompt(questions)
    return answer['value']


def project_confirmation(project):
    if not ask_confirm("Vuoi continuare sul progetto {}?".format(project)):
        sys.exit(-1)
